/* 
 * CsvEditor is "CSVFILE edit software".
 * 
 * Copyright (C) 2014  Isao Togo
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/******************************************************************************
 * TITLE           : CsvEditor
 * VERSION         : 1.0.0
 * LANGUAGE        : Java7(1.7.0 Update45)
 * ORIGINAL AUTHOR : 2009.06.21  I.TOGO  (Ver 0.1)   (JDK 1.5.0 beta2)
 * UPDATED         : 2014.01.08  I.TOGO  (Ver 1.0.0) (JDK 1.7.0 Update45)
 * UPDATED         : ****.**.**
 ******************************************************************************/
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.table.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/******************************************************************************
 *bd
 ******************************************************************************/
class CsvEditor extends JFrame implements ActionListener, MouseListener {

	private String				TITLE = "CsvEditor Ver 1.0.0";
	
	// ʃvpeB
	private CommonProperties	commonProperties;

	private JToolBar			toolBar;
	private JLabel				statusBar;
	private JTable				tblArea;
	private DefaultTableModel	tblModel;
	private JPopupMenu			popup;

	// t@CI_CAO
	private JFileChooser		fileChooser = new JFileChooser(new File(".").getAbsoluteFile());
	private JRadioButton		jrdoHeader;
	private JRadioButton		jrdoData;
	
	// R{{bNXiGR[hj
	private String[]			jcboEnc_items = {"Shift-JIS(Windows{)",
												 "Shift-JIS({)",
												 "EUC({)",
												 "JIS({)",
												 "Unicode(BigEndian)",
												 "UTF-8"
												};
	private JComboBox			jcboEnc = new JComboBox(jcboEnc_items);


	////////////////////////////////////////////////////////////////////////////
	//RXgN^
	////////////////////////////////////////////////////////////////////////////
	CsvEditor() {
		// XvbVXN[
		SplashScreen splash = null;
		Graphics2D graphic = null;
		Dimension size = null;
		splash = SplashScreen.getSplashScreen();
		graphic = (Graphics2D)splash.createGraphics();
		size = splash.getSize();
		if (splash != null) {
			graphic.setComposite(AlphaComposite.Clear);
			graphic.fillRect(0, 0, size.width, size.height);
			graphic.setPaintMode();
			splash.update();
			try {
				Thread.sleep(200);
			} catch (InterruptedException ex) {
				StringWriter stringWriter = new StringWriter();
				PrintWriter printWriter = new PrintWriter(stringWriter);
				ex.printStackTrace(printWriter);
				JOptionPane.showMessageDialog(this,  stringWriter.toString(), "xbZ[W", JOptionPane.WARNING_MESSAGE);
				return;
			}
		}

		// C
		String lookAndFeel = "";
		lookAndFeel = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";	//Windows
		try {
			UIManager.setLookAndFeel(lookAndFeel);
			SwingUtilities.updateComponentTreeUI(this);
		} catch (Exception ex) { 
		}
		setTitle(TITLE);

		// ʃvpeB
		commonProperties = new CommonProperties();
		commonProperties.clear();

		InitMenuBar();		// j[o[쐬
		InitToolBar();		// c[o[쐬
		InitStatusBar();	// Xe[^Xo[쐬
		InitTableArea();	// e[uGA쐬
		InitPopupMenu();	// |bvAbvj[쐬
		InitFileChooser();	// t@C_CAÕTupl쐬

		setDefaultCloseOperation(this.EXIT_ON_CLOSE);
		pack();

		// t[XN[ɕ\
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(((screen.width - getSize().width) / 2),
					((screen.height - getSize().height) / 2));

		setVisible(true);
	}

	////////////////////////////////////////////////////////////////////////////
	//j[o[쐬
	////////////////////////////////////////////////////////////////////////////
	void InitMenuBar() {

		// j[o[
		JMenuBar menuBar = new JMenuBar();
		getRootPane().setJMenuBar(menuBar);

		// [t@C]
		JMenu menuFile = new JMenu("t@C(F)");
		menuFile.setMnemonic('F');
		menuBar.add(menuFile);

		// [t@C]-[VK쐬]
		JMenuItem menuNew = new JMenuItem("VK쐬(N)");
		menuNew.setMnemonic('N');
		menuNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK));
		menuNew.setActionCommand("New");
		menuNew.addActionListener(this);
		menuFile.add(menuNew);

		// [t@C]-[J]
		JMenuItem menuOpen = new JMenuItem("J(O)...");
		menuOpen.setMnemonic('O');
		menuOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK));
		menuOpen.setActionCommand("Open");
		menuOpen.addActionListener(this);
		menuFile.add(menuOpen);

		// [t@C]-[㏑ۑ]
		JMenuItem menuSave = new JMenuItem("㏑ۑ(S)");
		menuSave.setMnemonic('S');
		menuSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK));
		menuSave.setActionCommand("Save");
		menuSave.addActionListener(this);
		menuFile.add(menuSave);

		// [t@C]-[Oĕۑ]
		JMenuItem menuSaveAs = new JMenuItem("Otĕۑ(A)...");
		menuSaveAs.setMnemonic('A');
		menuSaveAs.setActionCommand("SaveAs");
		menuSaveAs.addActionListener(this);
		menuFile.add(menuSaveAs);

		// [t@C]-[ҏWʂւ̒ǉǂݍ]
		JMenuItem menuOpenIns = new JMenuItem("ҏWʂւ̒ǉǂݍ");
		menuOpenIns.setActionCommand("OpenIns");
		menuOpenIns.addActionListener(this);
		menuFile.add(menuOpenIns);

		// [t@C]-Zp[^
		menuFile.addSeparator();

		// [t@C]-[JSON`ŏo(object)]
		JMenuItem menuJsonOut1 = new JMenuItem("JSON`ŏo(object)");
		menuJsonOut1.setActionCommand("JsonOut1");
		menuJsonOut1.addActionListener(this);
		menuFile.add(menuJsonOut1);

		// [t@C]-[JSON`ŏo(array)]
		JMenuItem menuJsonOut2 = new JMenuItem("JSON`ŏo(array)");
		menuJsonOut2.setActionCommand("JsonOut2");
		menuJsonOut2.addActionListener(this);
		menuFile.add(menuJsonOut2);

		// [t@C]-Zp[^
		menuFile.addSeparator();

		// [t@C]-[I]
		JMenuItem menuExit = new JMenuItem("I(X)");
		menuExit.setMnemonic('X');
		menuExit.setActionCommand("Exit");
		menuExit.addActionListener(this);
		menuFile.add(menuExit);

		// [ҏW]
		JMenu menuEdit = new JMenu("ҏW(E)");
		menuEdit.setMnemonic('E');
		menuBar.add(menuEdit);

		// [ҏW]-[s}]
		JMenuItem menuInsRow = new JMenuItem("ݕҏW̍sɂPs}");
		menuInsRow.setActionCommand("InsRow");
		menuInsRow.addActionListener(this);
		menuEdit.add(menuInsRow);

		// [ҏW]-[s폜]
		JMenuItem menuDelRow = new JMenuItem("ݕҏW̍sPs폜");
		menuDelRow.setActionCommand("DelRow");
		menuDelRow.addActionListener(this);
		menuEdit.add(menuDelRow);

		// [ҏW]-[sǉ]
		JMenuItem menuAddRow = new JMenuItem("\̈ԉɂPs}");
		menuAddRow.setActionCommand("AddRow");
		menuAddRow.addActionListener(this);
		menuEdit.add(menuAddRow);

		// [\]
		JMenu menuView = new JMenu("\(V)");
		menuView.setMnemonic('V');
		menuBar.add(menuView);

		// [\]-[c[o[]
		JCheckBoxMenuItem menuToolBar = new JCheckBoxMenuItem("c[o[(T)", true);
		menuToolBar.setMnemonic('T');
		menuToolBar.setActionCommand("ToolBar");
		menuToolBar.addActionListener(this);
		menuView.add(menuToolBar);

		// [\]-[Xe[^Xo[]
		JCheckBoxMenuItem menuStatusBar = new JCheckBoxMenuItem("Xe[^Xo[(S)", true);
		menuStatusBar.setMnemonic('S');
		menuStatusBar.setActionCommand("StatusBar");
		menuStatusBar.addActionListener(this);
		menuView.add(menuStatusBar);

		// [wv]
		JMenu menuHelp = new JMenu("wv(H)");
		menuHelp.setMnemonic('H');
		menuBar.add(menuHelp);

		// [wv]-[CsvEditorɂ]
		JMenuItem menuAbout = new JMenuItem("CsvEditorɂ(A)...");
		menuAbout.setMnemonic('A');
		menuAbout.setActionCommand("About");
		menuAbout.addActionListener(this);
		menuHelp.add(menuAbout);

		// [wv]-Zp[^
		menuHelp.addSeparator();

		// [wv]-[GNU General Public License version 3 (GPLv3)]
		JMenuItem menuAboutGPL = new JMenuItem("GNU General Public License version 3 (GPLv3)");
		menuAboutGPL.setActionCommand("AboutGPL");
		menuAboutGPL.addActionListener(this);
		menuHelp.add(menuAboutGPL);

		// [wv]-[GNU Lesser General Public License Version 2.1(LGPL2.1)]
		JMenuItem menuAboutLGPL = new JMenuItem("GNU Lesser General Public License Version 2.1(LGPL2.1)");
		menuAboutLGPL.setActionCommand("AboutLGPL");
		menuAboutLGPL.addActionListener(this);
		menuHelp.add(menuAboutLGPL);

		// [wv]-[NOTICE]
		JMenuItem menuAboutNOTICE = new JMenuItem("NOTICE");
		menuAboutNOTICE.setActionCommand("AboutNOTICE");
		menuAboutNOTICE.addActionListener(this);
		menuHelp.add(menuAboutNOTICE);

	}

	////////////////////////////////////////////////////////////////////////////
	//c[o[쐬
	////////////////////////////////////////////////////////////////////////////
	void InitToolBar() {
		toolBar = new JToolBar();
		toolBar.setFloatable(false);

		JButton btnNew = new JButton(new ImageIcon(getClass().getClassLoader().getResource("Resources/Images/filenew.png")));
		JButton btnOpen = new JButton(new ImageIcon(getClass().getClassLoader().getResource("Resources/Images/fileopen.png")));
		JButton btnSave = new JButton(new ImageIcon(getClass().getClassLoader().getResource("Resources/Images/filesave.png")));
		JButton btnSaveAs = new JButton(new ImageIcon(getClass().getClassLoader().getResource("Resources/Images/filesaveas.png")));

		btnNew.setToolTipText("VK쐬");
		btnOpen.setToolTipText("J");
		btnSave.setToolTipText("㏑ۑ");
		btnSaveAs.setToolTipText("Otĕۑ");

		btnNew.setActionCommand("New");
		btnOpen.setActionCommand("Open");
		btnSave.setActionCommand("Save");
		btnSaveAs.setActionCommand("SaveAs");

		btnNew.addActionListener(this);
		btnOpen.addActionListener(this);
		btnSave.addActionListener(this);
		btnSaveAs.addActionListener(this);

		toolBar.add(btnNew);
		toolBar.add(btnOpen);
		toolBar.add(btnSave);
		toolBar.add(btnSaveAs);

		getContentPane().add(toolBar, BorderLayout.NORTH);
	}

	////////////////////////////////////////////////////////////////////////////
	//Xe[^Xo[쐬
	////////////////////////////////////////////////////////////////////////////
	void InitStatusBar() {
		statusBar = new JLabel("@");
		getContentPane().add(statusBar, BorderLayout.SOUTH);
	}

	////////////////////////////////////////////////////////////////////////////
	//e[uGA쐬
	////////////////////////////////////////////////////////////////////////////
	void InitTableArea() {

		tblModel = new DefaultTableModel();
		tblArea = new JTable(tblModel);

		tblArea.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		tblArea.addMouseListener(this);

		// ZPʂł̑I\ɂ
		tblArea.setColumnSelectionAllowed(false);
		tblArea.setRowSelectionAllowed(false);

		// VONbNi}EXNbN񐔁PjŃZҏW\Ƃ
		DefaultCellEditor ce = (DefaultCellEditor)tblArea.getDefaultEditor(Object.class);
		ce.setClickCountToStart(1);

		JScrollPane scrollpane = new JScrollPane(tblArea);
		scrollpane.setPreferredSize(new Dimension(800, 500));

		getContentPane().add(scrollpane, BorderLayout.CENTER);
	}

	////////////////////////////////////////////////////////////////////////////
	//|bvAbvj[쐬
	////////////////////////////////////////////////////////////////////////////
	void InitPopupMenu() {

		popup = new JPopupMenu();
		JMenuItem menuA = new JMenuItem("s}");
		JMenuItem menuB = new JMenuItem("s폜");
		JMenuItem menuC = new JMenuItem("\̈ԉɂPs}");

		menuA.setActionCommand("InsRow");
		menuB.setActionCommand("DelRow");
		menuC.setActionCommand("AddRow");

		menuA.addActionListener(this);
		menuB.addActionListener(this);
		menuC.addActionListener(this);

		popup.add(menuA);
		popup.add(menuB);
		popup.add(menuC);
	}

	////////////////////////////////////////////////////////////////////////////
	//t@C_CAÕTupl쐬
	////////////////////////////////////////////////////////////////////////////
	void InitFileChooser() {

		// GR[h̎w
		JLabel jlblEnc = new JLabel("GR[h",JLabel.LEFT);
		jlblEnc.setBounds(10,10,80,20);
		jlblEnc.setFont(new Font("lr SVbN",Font.BOLD,12));

		jcboEnc.setEditable(false);
		jcboEnc.setBounds(90,10,180,20);
		jcboEnc.setFont(new Font("lr SVbN",Font.PLAIN,12));

		// bruwb_̎w
		JLabel jlblHdr = new JLabel("bruwb_",JLabel.LEFT);
		jlblHdr.setBounds(10,40,120,20);
		jlblHdr.setFont(new Font("lr SVbN",Font.BOLD,12));

		// bruwb_̎wiLj
		jrdoHeader = new JRadioButton("L",true);
		jrdoHeader.setBounds(100,40,50,20);
		jrdoHeader.setFont(new Font("lr SVbN",Font.PLAIN,12));
		// bruwb_̎wij
		jrdoData = new JRadioButton("",false);
		jrdoData.setBounds(150,40,50,20);
		jrdoData.setFont(new Font("lr SVbN",Font.PLAIN,12));
		// bruwb_w胉WI{^O[v
		ButtonGroup group1 = new ButtonGroup();
		group1.add(jrdoHeader);
		group1.add(jrdoData);

		JPanel subPanel = new JPanel();
		subPanel.setLayout(null);
		subPanel.setPreferredSize(new Dimension(290, 80));
		subPanel.add(jcboEnc);
		subPanel.add(jlblEnc);
		subPanel.add(jlblHdr);
		subPanel.add(jrdoHeader);
		subPanel.add(jrdoData);
		fileChooser.setAccessory(subPanel);
	}

	////////////////////////////////////////////////////////////////////////////
	//Cxg
	////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent ae) {
		String cmd = ae.getActionCommand();
		if (cmd.equals("New")) {
			OnNew();
		} else if (cmd.equals("Open")) {
			OnOpen();
		} else if (cmd.equals("Save")) {
			OnSave();
		} else if (cmd.equals("SaveAs")) {
			OnSaveAs();
		} else if (cmd.equals("OpenIns")) {
			OnOpenIns();
		} else if (cmd.equals("JsonOut1")) {
			OnJsonOut1();
		} else if (cmd.equals("JsonOut2")) {
			OnJsonOut2();
		} else if (cmd.equals("Exit")) {
			OnExit();
		} else if (cmd.equals("InsRow")) {
			OnInsRow();
		} else if (cmd.equals("DelRow")) {
			OnDelRow();
		} else if (cmd.equals("AddRow")) {
			OnAddRow();
		} else if (cmd.equals("ToolBar")) {
			OnToolBar(((JCheckBoxMenuItem)ae.getSource()).getState());
		} else if (cmd.equals("StatusBar")) {
			OnStatusBar(((JCheckBoxMenuItem)ae.getSource()).getState());
		} else if (cmd.equals("About")) {
			OnAbout();
		} else if (cmd.equals("AboutGPL")) {
			OnAboutGPL();
		} else if (cmd.equals("AboutLGPL")) {
			OnAboutLGPL();
		} else if (cmd.equals("AboutNOTICE")) {
			OnAboutNOTICE();
		} else {
			System.out.println(cmd);
		}
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
		if (e.isPopupTrigger()) {
			popup.show(e.getComponent(), e.getX(), e.getY());
		}
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	// [t@C]-[VK쐬]
	void OnNew() {

		// ʕ\
		NewFile d = new NewFile(new JFrame(), commonProperties, tblModel);
		if (d.getReturn()) {
			// [VK쐬]{^
			fileChooser.setSelectedFile(null);
			setTitle(TITLE + " - VKt@C");
			// Xe[^Xo[
			statusBar.setText("@");
		}
	}

	// [t@C]-[J]
	void OnOpen() {

		// t@CI_CAO̕\
		fileChooser.setDialogTitle("J");
		int returnVal = fileChooser.showOpenDialog(this);
		try {
			if (returnVal == JFileChooser.APPROVE_OPTION) {

				tblModel.setRowCount(0);	// \̑Ss폜
				tblModel.setColumnCount(0);	// \̑S폜

				// 쐬[hi0:XVj
				commonProperties.setCreateMode("0");

				// _CAOőIt@C擾ă^Cgɕ\
				File file = fileChooser.getSelectedFile();
				setTitle(TITLE + " - " + file.getAbsolutePath());

				// Xg[̍쐬Abv
				InputStreamReader in = null;
				if (jcboEnc.getSelectedItem().equals("Shift-JIS(Windows{)")) {
					in = new InputStreamReader(new FileInputStream(file), "MS932");
				} else if (jcboEnc.getSelectedItem().equals("Shift-JIS({)")) {
					in = new InputStreamReader(new FileInputStream(file), "SJIS");
				} else if (jcboEnc.getSelectedItem().equals("EUC({)")) {
					in = new InputStreamReader(new FileInputStream(file), "EUC_JP");
				} else if (jcboEnc.getSelectedItem().equals("JIS({)")) {
					in = new InputStreamReader(new FileInputStream(file), "ISO2022JP");
				} else if (jcboEnc.getSelectedItem().equals("Unicode(BigEndian)")) {
					in = new InputStreamReader(new FileInputStream(file), "UnicodeBig");
				} else if (jcboEnc.getSelectedItem().equals("UTF-8")) {
					in = new InputStreamReader(new FileInputStream(file), "UTF8");
				}
				// <ʃvpeB>FGR[h
				commonProperties.setEncode(jcboEnc.getSelectedItem().toString());
				// <ʃvpeB>Fbruwb_
				if (jrdoHeader.isSelected()) {
					// 1:wb_L
					commonProperties.setCsvHeader("1");
				} else {
					// 0:wb_
					commonProperties.setCsvHeader("0");
				}
				// <ʃvpeB>Ft@C
				commonProperties.setFilePath(file.getAbsolutePath());
				
				// brut@C荞
				CsvFileRead(in, 0);
				
				// Xg[
				in.close();
			}
		} catch(Exception ex) {
			ex.printStackTrace();
		}

	}

	// [t@C]-[㏑ۑ]
	void OnSave() {

		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "f[^P܂B" + System.getProperty("line.separator") + "brut@Cւ̕ۑ͍s܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}

		if (commonProperties.getCreateMode() == null) {
			OnSaveAs();
			return;
		}
		if (commonProperties.getCreateMode().equalsIgnoreCase("1")) {
			OnSaveAs();
		} else {
			try {
				// brut@Co
				CsvFileWrite();
			} catch(Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	// [t@C]-[Otĕۑ]
	void OnSaveAs() {

		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "f[^P܂B" + System.getProperty("line.separator") + "brut@Cւ̕ۑ͍s܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}

		// t@C
		if (commonProperties.getFilePath() != null) {
			fileChooser.setSelectedFile(new File(commonProperties.getFilePath()));
		}

		// GR[h
		if (commonProperties.getEncode() != null) {
			jcboEnc.setSelectedItem(commonProperties.getEncode());
		}

		// bruwb_i0:A1:Lj
		if (commonProperties.getCsvHeader().equalsIgnoreCase("1")) {
			jrdoHeader.setSelected(true);	// wb_L
		} else {
			jrdoData.setSelected(true);		// wb_
		}

		// t@CI_CAO̕\
		fileChooser.setDialogTitle("Otĕۑ");
		int returnVal = fileChooser.showSaveDialog(this);

		try {
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fileChooser.getSelectedFile();
				// w肵t@C̑݃`FbNs
				if (file.exists()) {
					// Ƀt@C݂ꍇ́A㏑邩ǂ̊mF_CAO\
					int rc = JOptionPane.showConfirmDialog(null, file.getAbsolutePath() + System.getProperty("line.separator") + "̃t@C͊ɑ݂܂B" + System.getProperty("line.separator") + System.getProperty("line.separator") + "㏑܂H", "Otĕۑ", JOptionPane.YES_NO_OPTION);
					if (rc == JOptionPane.YES_OPTION) {
						// bruwb_i0:A1:Lj
						if (jrdoHeader.isSelected()) {
							commonProperties.setCsvHeader("1");
						} else {
							commonProperties.setCsvHeader("0");
						}
						// GR[h
						commonProperties.setEncode(jcboEnc.getSelectedItem().toString());
						// t@C
						commonProperties.setFilePath(file.getAbsolutePath());
						
						// brut@Co
						CsvFileWrite();
					}
				} else {
					// bruwb_i0:A1:Lj
					if (jrdoHeader.isSelected()) {
						commonProperties.setCsvHeader("1");
					} else {
						commonProperties.setCsvHeader("0");
					}
					// GR[h
					commonProperties.setEncode(jcboEnc.getSelectedItem().toString());
					// t@C
					commonProperties.setFilePath(file.getAbsolutePath());

					// brut@Co
					CsvFileWrite();
				}
			}
		} catch(Exception ex) {
			ex.printStackTrace();
 		}
	}

	// [t@C]-[ҏWʂւ̒ǉǂݍ]
	void OnOpenIns() {

		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "ҏW̃f[^P܂B" + System.getProperty("line.separator") + "t@C(F)J(O)łbrut@C̎荞݂sĂB", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}

		// GR[h̎w
		JLabel jlblEnc = new JLabel("GR[h",JLabel.LEFT);
		jlblEnc.setBounds(10,10,80,20);
		jlblEnc.setFont(new Font("lr SVbN",Font.BOLD,12));

		jcboEnc.setEditable(false);
		jcboEnc.setBounds(90,10,180,20);
		jcboEnc.setFont(new Font("lr SVbN",Font.PLAIN,12));


		JPanel subPanel = new JPanel();
		subPanel.setLayout(null);
		subPanel.setPreferredSize(new Dimension(290, 80));
		subPanel.add(jcboEnc);
		subPanel.add(jlblEnc);
		fileChooser.setAccessory(subPanel);

		// t@CI_CAO̕\
		fileChooser.setDialogTitle("ҏWʂւ̒ǉǂݍ");
		int returnVal = fileChooser.showOpenDialog(this);
		try {
			if (returnVal == JFileChooser.APPROVE_OPTION) {

				File file = fileChooser.getSelectedFile();

				// Xg[̍쐬Abv
				InputStreamReader in = null;
				if (jcboEnc.getSelectedItem().equals("Shift-JIS(Windows{)")) {
					in = new InputStreamReader(new FileInputStream(file), "MS932");
				} else if (jcboEnc.getSelectedItem().equals("Shift-JIS({)")) {
					in = new InputStreamReader(new FileInputStream(file), "SJIS");
				} else if (jcboEnc.getSelectedItem().equals("EUC({)")) {
					in = new InputStreamReader(new FileInputStream(file), "EUC_JP");
				} else if (jcboEnc.getSelectedItem().equals("JIS({)")) {
					in = new InputStreamReader(new FileInputStream(file), "ISO2022JP");
				} else if (jcboEnc.getSelectedItem().equals("Unicode(BigEndian)")) {
					in = new InputStreamReader(new FileInputStream(file), "UnicodeBig");
				} else if (jcboEnc.getSelectedItem().equals("UTF-8")) {
					in = new InputStreamReader(new FileInputStream(file), "UTF8");
				}
				
				// brut@C荞
				CsvFileRead(in, 1);
				
				// Xg[
				in.close();
			}
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}

	// [t@C]-[JSON`ŏo(object)]
	void OnJsonOut1() {
		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "f[^P܂B" + System.getProperty("line.separator") + "irnm`ł̏o͍͂s܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}
		// t@C
		if (commonProperties.getFilePath() != null) {
			fileChooser.setSelectedFile(new File(commonProperties.getFilePath()));
		}
		// GR[h
		if (commonProperties.getEncode() != null) {
			jcboEnc.setSelectedItem(commonProperties.getEncode());
		}
		// bruwb_i0:A1:Lj
		if (commonProperties.getCsvHeader().equalsIgnoreCase("1")) {
			jrdoHeader.setSelected(true);	// wb_L
		} else {
			jrdoData.setSelected(true);		// wb_
		}

		// t@CI_CAO̕\
		fileChooser.setDialogTitle("JSON`ŏo(object)");
		int returnVal = fileChooser.showSaveDialog(this);
		try {
			if (returnVal == JFileChooser.APPROVE_OPTION) {

				File file = fileChooser.getSelectedFile();
				// w肵t@C̑݃`FbNs
				if (file.exists()) {
					// Ƀt@C݂ꍇ́A㏑邩ǂ̊mF_CAO\
					int rc = JOptionPane.showConfirmDialog(null, file.getAbsolutePath() + System.getProperty("line.separator") + "̃t@C͊ɑ݂܂B" + System.getProperty("line.separator") + System.getProperty("line.separator") + "㏑܂H", "JSON`ŏóiobjectj", JOptionPane.YES_NO_OPTION);
					if (rc == JOptionPane.YES_OPTION) {

						// bruwb_i0:A1:Lj
						if (jrdoHeader.isSelected()) {
							commonProperties.setCsvHeader("1");
						} else {
							commonProperties.setCsvHeader("0");
						}
						// GR[h
						commonProperties.setEncode(jcboEnc.getSelectedItem().toString());
						// t@C
						commonProperties.setFilePath(file.getAbsolutePath());
						
						// irnm`Ńt@Co(object)
						JsonFileWrite1();
					}
				} else {

					// bruwb_i0:A1:Lj
					if (jrdoHeader.isSelected()) {
						commonProperties.setCsvHeader("1");
					} else {
						commonProperties.setCsvHeader("0");
					}
					// GR[h
					commonProperties.setEncode(jcboEnc.getSelectedItem().toString());
					// t@C
					commonProperties.setFilePath(file.getAbsolutePath());

					// irnm`Ńt@Co(object)
					JsonFileWrite1();
				}
			}
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}

	// [t@C]-[JSON`ŏo(array)]
	void OnJsonOut2() {
		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "f[^P܂B" + System.getProperty("line.separator") + "irnm`ł̏o͍͂s܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}
		// t@C
		if (commonProperties.getFilePath() != null) {
			fileChooser.setSelectedFile(new File(commonProperties.getFilePath()));
		}
		// GR[h
		if (commonProperties.getEncode() != null) {
			jcboEnc.setSelectedItem(commonProperties.getEncode());
		}
		// bruwb_i0:A1:Lj
		if (commonProperties.getCsvHeader().equalsIgnoreCase("1")) {
			jrdoHeader.setSelected(true);	// wb_L
		} else {
			jrdoData.setSelected(true);		// wb_
		}

		// t@CI_CAO̕\
		fileChooser.setDialogTitle("JSON`ŏo(array)");
		int returnVal = fileChooser.showSaveDialog(this);
		try {
			if (returnVal == JFileChooser.APPROVE_OPTION) {

				File file = fileChooser.getSelectedFile();
				// w肵t@C̑݃`FbNs
				if (file.exists()) {
					// Ƀt@C݂ꍇ́A㏑邩ǂ̊mF_CAO\
					int rc = JOptionPane.showConfirmDialog(null, file.getAbsolutePath() + System.getProperty("line.separator") + "̃t@C͊ɑ݂܂B" + System.getProperty("line.separator") + System.getProperty("line.separator") + "㏑܂H", "JSON`ŏóiarrayj", JOptionPane.YES_NO_OPTION);
					if (rc == JOptionPane.YES_OPTION) {

						// bruwb_i0:A1:Lj
						if (jrdoHeader.isSelected()) {
							commonProperties.setCsvHeader("1");
						} else {
							commonProperties.setCsvHeader("0");
						}
						// GR[h
						commonProperties.setEncode(jcboEnc.getSelectedItem().toString());
						// t@C
						commonProperties.setFilePath(file.getAbsolutePath());
						
						// irnm`Ńt@Co(array)
						JsonFileWrite2();
					}
				} else {

					// bruwb_i0:A1:Lj
					if (jrdoHeader.isSelected()) {
						commonProperties.setCsvHeader("1");
					} else {
						commonProperties.setCsvHeader("0");
					}
					// GR[h
					commonProperties.setEncode(jcboEnc.getSelectedItem().toString());
					// t@C
					commonProperties.setFilePath(file.getAbsolutePath());

					// irnm`Ńt@Co(array)
					JsonFileWrite2();
				}
			}
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}

	// [t@C]-[I]
	void OnExit() {
		System.exit(0);
	}

	// [ҏW]-[s}]
	void OnInsRow() {
		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "ҏWf[^܂B" + System.getProperty("line.separator") + "܂͂bruf[^̓ǂݍ݂sĂB" , "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}
		Object[] cell = null;
		// ݑIĂZ̍sԍ擾
		int row = tblArea.getSelectedRow();
		if (row < 0) {
			// IĂsꍇ͌xbZ[W\
			JOptionPane.showMessageDialog(this, "sIĂ܂B" + System.getProperty("line.separator") + "}s̃ZNbNĂB", "xbZ[W", JOptionPane.WARNING_MESSAGE);
		} else {
			// Z̕ҏW~i}󔒍sɕҏW̃Z̒lݒ肳Ă܂߁j
			if(tblArea.isEditing()) {
				tblArea.getCellEditor().stopCellEditing();
			}
			// ݑIĂsɋ󔒍s}
			tblModel.insertRow(row, cell);
		}
	}

	// [ҏW]-[s폜]
	void OnDelRow() {
		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "ҏWf[^܂B" + System.getProperty("line.separator") + "܂͂bruf[^̓ǂݍ݂sĂB" , "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}
		// ݑIĂZ̍sԍ擾
		int row = tblArea.getSelectedRow();
		if (row < 0) {
			// IĂsꍇ͌xbZ[W\
			JOptionPane.showMessageDialog(this, "sIĂ܂B" + System.getProperty("line.separator") + "폜s̃ZNbNĂB", "xbZ[W", JOptionPane.WARNING_MESSAGE);
		} else {
			// Z̕ҏW~i폜̃JgsɕҏW̃Z̒lݒ肳Ă܂߁j
			if(tblArea.isEditing()) {
				tblArea.getCellEditor().stopCellEditing();
			}
			// ݑIĂs폜
			tblModel.removeRow(row);
		}
	}

	// [ҏW]-[\̈ԉɋ󔒍s}]
	void OnAddRow() {
		if (tblModel.getRowCount() == 0) {
			JOptionPane.showMessageDialog(this, "ҏWf[^܂B" + System.getProperty("line.separator") + "܂͂bruf[^̓ǂݍ݂sĂB" , "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}
		Object[] cell = null;
		// Z̕ҏW~i}󔒍sɕҏW̃Z̒lݒ肳Ă܂߁j
		if(tblArea.isEditing()) {
			tblArea.getCellEditor().stopCellEditing();
		}
		tblModel.addRow(cell);
	}

	// [\]-[c[o[]
	void OnToolBar(boolean b) {
		toolBar.setVisible(b);
	}

	// [\]-[Xe[^Xo[]
	void OnStatusBar(boolean b) {
		statusBar.setVisible(b);
	}

	////////////////////////////////////////////////////////////////////////////
	//[wv]-[CsvEditorɂ]
	////////////////////////////////////////////////////////////////////////////
	void OnAbout() {
		FormAbout d = new FormAbout(new JFrame());
	}
	////////////////////////////////////////////////////////////////////////////
	//[wv]-[GNU General Public License version 3 (GPLv3)]
	////////////////////////////////////////////////////////////////////////////
	void OnAboutGPL() {
		FormAboutGpl d = new FormAboutGpl(new JFrame());
	}
	////////////////////////////////////////////////////////////////////////////
	//[wv]-[GNU Lesser General Public License Version 2.1(LGPL2.1)]
	////////////////////////////////////////////////////////////////////////////
	void OnAboutLGPL() {
		FormAboutLgpl d = new FormAboutLgpl(new JFrame());
	}
	////////////////////////////////////////////////////////////////////////////
	//[wv]-[NOTICE]
	////////////////////////////////////////////////////////////////////////////
	void OnAboutNOTICE() {
		FormNOTICE d = new FormNOTICE(new JFrame());
	}

	////////////////////////////////////////////////////////////////////////////
	//brut@C荞
	////////////////////////////////////////////////////////////////////////////
	void CsvFileRead(InputStreamReader in, int insMode) {
		try {
			setCursor(new Cursor(Cursor.WAIT_CURSOR));		// ҂J[\

			if (insMode == 0) {
				// Xe[^Xo[
				statusBar.setText("m" + commonProperties.getEncode() + "n");
			}

			String inString = null;
			Object[] cell = null;
			BufferedReader br = new BufferedReader(in);
			
			if ((inString = br.readLine()) != null) {
				
				// t@Cǂݍ񂾂Ps̖̕ɁuCvt
				inString = inString.replaceAll("(?:" + System.getProperty("line.separator") + "|[" + System.getProperty("line.separator") + "])?$", ",");

				// t@Cǂݍ񂾂Ps̕uCvŕ
				Pattern pattern = Pattern.compile("(\"[^\"]*(?:\"\"[^\"]*)*\"|[^,]*),");
				Matcher matcher = null;
				matcher = pattern.matcher(inString);

				// bruwb_
				if (insMode == 0) {
					// iinsMode=0:ʏ̓ǂݍ݁j
					if (commonProperties.getCsvHeader().equalsIgnoreCase("1")) {
						// bruwb_i1:Lj
						while (matcher.find()) {
							// K\p^[Ƀ}b`O[v擾
							String csvValue = matcher.group(1);
							// 擪Ɩ́u"v菜i:"ABC"ABCɂj
							csvValue = csvValue.replaceAll("^\"(.*)\"$", "$1");
							//u""vu"vɕϊ
							csvValue = csvValue.replaceAll("\"\"", "\"");
							tblModel.addColumn(csvValue);
						}
					} else {
						// bruwb_i0:j
						if (matcher.groupCount() > 0) {
							// uCvŕbruڐ擾
							int i = 0;
							while (matcher.find()) {
								// bruwb_̏ꍇ́ǍoAԂɂ
								i = i + 1;
								tblModel.addColumn(Integer.toString(i));
							}
							cell = new Object[i];

							i = 0;
							matcher = pattern.matcher(inString);
							while (matcher.find()) {
								if (i < tblModel.getColumnCount()) {
									// K\p^[Ƀ}b`O[v擾
									String csvValue = matcher.group(1);
									// 擪Ɩ́u"v菜i:"ABC"ABCɂj
									csvValue = csvValue.replaceAll("^\"(.*)\"$", "$1");
									// u""vu"vɕϊ
									csvValue = csvValue.replaceAll("\"\"", "\"");
									cell[i] = csvValue;
									i = i + 1;
								}
							}
							tblModel.addRow(cell);
						}
					}
				} else {
					// iinsMode=1:ҏWʒuւ̒ǉǂݍ݁j
					if (matcher.groupCount() > 0) {
						// uCvŕbruڐ擾
						cell = new Object[tblModel.getColumnCount()];

						int i = 0;
						matcher = pattern.matcher(inString);
						while (matcher.find()) {
							if (i < tblModel.getColumnCount()) {
								// K\p^[Ƀ}b`O[v擾
								String csvValue = matcher.group(1);
								// 擪Ɩ́u"v菜i:"ABC"ABCɂj
								csvValue = csvValue.replaceAll("^\"(.*)\"$", "$1");
								// u""vu"vɕϊ
								csvValue = csvValue.replaceAll("\"\"", "\"");
								cell[i] = csvValue;
								i = i + 1;
							}
						}
						tblModel.addRow(cell);
					}
				}

				// CSVf[^Qsڈȍ~if[^j
				while( (inString = br.readLine()) != null) {
					// t@Cǂݍ񂾂Ps̖̕ɁuCvt
					inString = inString.replaceAll("(?:" + System.getProperty("line.separator") + "|[" + System.getProperty("line.separator") + "])?$", ",");

					// t@Cǂݍ񂾂Ps̕uCvŕ
					matcher = pattern.matcher(inString);

					if (matcher.groupCount() > 0) {
						// uCvŕbruڐ擾
						int i = 0;
						while (matcher.find()) {
							i = i + 1;
						}
						cell = new Object[i];

						i = 0;
						matcher = pattern.matcher(inString);
						while (matcher.find()) {
							if (i < tblModel.getColumnCount()) {
								// K\p^[Ƀ}b`O[v擾
								String csvValue = matcher.group(1);
								// 擪Ɩ́u"v菜i:"ABC"ABCɂj
								csvValue = csvValue.replaceAll("^\"(.*)\"$", "$1");
								// u""vu"vɕϊ
								csvValue = csvValue.replaceAll("\"\"", "\"");
								cell[i] = csvValue;
								i = i + 1;
							}
						}
						tblModel.addRow(cell);
					}
				}
				pack();
				setVisible(true);
				
				Thread.sleep(5000);
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						resizeColumnWidth();
						tblArea.revalidate();
					}
				});
				setCursor(new Cursor(Cursor.DEFAULT_CURSOR));	// ftHgJ[\
			
			}
			
		} catch(Exception ex) {
			ex.printStackTrace();
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));	// ftHgJ[\
		}
	}

	////////////////////////////////////////////////////////////////////////////
	//brut@Co
	////////////////////////////////////////////////////////////////////////////
	void CsvFileWrite() {
		try {

			// t@C^Cgɕ\
			setTitle(TITLE + " - " + commonProperties.getFilePath());
			// Xe[^Xo[
			statusBar.setText("m" + commonProperties.getEncode() + "n");

			// t@C
			if (commonProperties.getFilePath() != null) {
				fileChooser.setSelectedFile(new File(commonProperties.getFilePath()));
			}

			// Xg[̍쐬Abv
			OutputStreamWriter out = null;
			if (commonProperties.getEncode().equals("Shift-JIS(Windows{)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "MS932");
			} else if (commonProperties.getEncode().equals("Shift-JIS({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "SJIS");
			} else if (commonProperties.getEncode().equals("EUC({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "EUC_JP");
			} else if (commonProperties.getEncode().equals("JIS({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "ISO2022JP");
			} else if (commonProperties.getEncode().equals("Unicode(BigEndian)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "UnicodeBig");
			} else if (commonProperties.getEncode().equals("UTF-8")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "UTF8");
			}
			// bruwb_L̏ꍇ̓wb_o͂ibrut@CPsڂ̏ój
			if (commonProperties.getCsvHeader().equalsIgnoreCase("1")) {
				StringBuffer sbBuf = new StringBuffer();
				for (int j=0; j<tblModel.getColumnCount(); j++) {
					String strCol = tblModel.getColumnName(j);
					if (strCol == null) {
						sbBuf.append("");
					} else {
						if ((strCol.indexOf(",") >= 0) ||
							(strCol.indexOf("\"") >= 0)) {
							// Z̓eɁuCv܂́uhv܂܂ꍇ́A
							// 擪Ɩuhvň͂ށB
							sbBuf.append("\"");
							for (int k=0; k<strCol.length(); k++) {
								sbBuf.append(strCol.charAt(k));
								if (strCol.charAt(k) == '"') {
									sbBuf.append("\"");
								}
							}
							sbBuf.append("\"");
						} else {
							// Z̓eɁuCv܂́uhv܂܂Ȃꍇ
							// 擪Ɩuhvň͂ޕKvȂ̂ŁA
							// Z̓ê܂܂bruɏo͂B
							sbBuf.append(strCol);
						}
					}
					if (j<(tblModel.getColumnCount() - 1)) {
						sbBuf.append(",");
					}
				}
				sbBuf.append(System.getProperty("line.separator"));
				out.write(sbBuf.toString());
			}
			// Xvbh̊eZ̓et@Cɏo͂ibrut@CQsڈȍ~̏ój
			for (int i=0; i<tblModel.getRowCount(); i++) {
				StringBuffer sbBuf = new StringBuffer();
				for (int j=0; j<tblModel.getColumnCount(); j++) {
					Object objCellVal = tblModel.getValueAt(i,j);
					if (objCellVal == null) {
						sbBuf.append("");
					} else {
						if ((objCellVal.toString().indexOf(",") >= 0) ||
							(objCellVal.toString().indexOf("\"") >= 0)) {
							// Z̓eɁuCv܂́uhv܂܂ꍇ́A
							// 擪Ɩuhvň͂ށB
							sbBuf.append("\"");
							for (int k=0; k<objCellVal.toString().length(); k++) {
								sbBuf.append(objCellVal.toString().charAt(k));
								if (objCellVal.toString().charAt(k) == '"') {
									sbBuf.append("\"");
								}
							}
							sbBuf.append("\"");
						} else {
							// Z̓eɁuCv܂́uhv܂܂Ȃꍇ
							// 擪Ɩuhvň͂ޕKvȂ̂ŁA
							// Z̓ê܂܂bruɏo͂B
							sbBuf.append(objCellVal.toString());
						}
					}
					if (j<(tblModel.getColumnCount() - 1)) {
						sbBuf.append(",");
					}
				}
				sbBuf.append(System.getProperty("line.separator"));
				out.write(sbBuf.toString());
			}
			// Xg[
			out.close();
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}

	////////////////////////////////////////////////////////////////////////////
	//irnm`Ńt@Cóij
	////////////////////////////////////////////////////////////////////////////
	void JsonFileWrite1() {
		try {

			// t@C^Cgɕ\
			setTitle(TITLE + " - " + commonProperties.getFilePath());
			// Xe[^Xo[
			statusBar.setText("m" + commonProperties.getEncode() + "n");

			// t@C
			if (commonProperties.getFilePath() != null) {
				fileChooser.setSelectedFile(new File(commonProperties.getFilePath()));
			}

			// Xg[̍쐬Abv
			OutputStreamWriter out = null;
			if (commonProperties.getEncode().equals("Shift-JIS(Windows{)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "MS932");
			} else if (commonProperties.getEncode().equals("Shift-JIS({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "SJIS");
			} else if (commonProperties.getEncode().equals("EUC({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "EUC_JP");
			} else if (commonProperties.getEncode().equals("JIS({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "ISO2022JP");
			} else if (commonProperties.getEncode().equals("Unicode(BigEndian)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "UnicodeBig");
			} else if (commonProperties.getEncode().equals("UTF-8")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "UTF8");
			}
			// bruwb_L̏ꍇ̓wb_o͂ibrut@CPsڂ̏ój
			if (commonProperties.getCsvHeader().equalsIgnoreCase("1")) {

				StringBuffer sbBuf = new StringBuffer();
				sbBuf.append("{");
				sbBuf.append(System.getProperty("line.separator"));
				out.write(sbBuf.toString());

				for (int j=0; j<tblModel.getColumnCount(); j++) {

					sbBuf.setLength(0);

					String strCol = tblModel.getColumnName(j);
					if (strCol == null) {
						sbBuf.append("\"\": [");
					} else {
						if ((strCol.indexOf("\"") >= 0) ||
							(strCol.indexOf("\\") >= 0)) {
							// Z̓eɁuhv܂́uv܂܂ꍇ́A
							// ̐擪ɁuvtB
							sbBuf.append("\"");
							for (int m=0; m<strCol.length(); m++) {
								if (strCol.charAt(m) == '"') {
									sbBuf.append("\\\"");
								} else if (strCol.charAt(m) == '\\') {
									sbBuf.append("\\\\");
								} else {
									sbBuf.append(strCol.charAt(m));
								}
							}
							sbBuf.append("\": [");
						} else {
							// JSON`̏ꍇ͐擪Ɩuhvň͂ށB
							sbBuf.append("\"");
							sbBuf.append(strCol);
							sbBuf.append("\": [");
						}
					}
					out.write(sbBuf.toString());
					sbBuf.setLength(0);

					for (int i=0; i<tblModel.getRowCount(); i++) {
						Object objCellVal = tblModel.getValueAt(i,j);
						if (objCellVal == null) {
							sbBuf.append("\"\"");
						} else {
							if ((objCellVal.toString().indexOf("\"") >= 0) ||
								(objCellVal.toString().indexOf("\\") >= 0)) {
								// Z̓eɁuhv܂́uv܂܂ꍇ́A
								// ̐擪ɁuvtB
								sbBuf.append("\"");
								for (int k=0; k<objCellVal.toString().length(); k++) {
									if (objCellVal.toString().charAt(k) == '"') {
										sbBuf.append("\\\"");
									} else if (objCellVal.toString().charAt(k) == '\\') {
										sbBuf.append("\\\\");
									} else {
										sbBuf.append(objCellVal.toString().charAt(k));
									}
								}
								sbBuf.append("\"");
							} else {
								// JSON`̏ꍇ͐擪Ɩuhvň͂ށB
								sbBuf.append("\"");
								sbBuf.append(objCellVal.toString());
								sbBuf.append("\"");
							}
						}
						if (i<(tblModel.getRowCount() - 1)) {
							sbBuf.append(",");
							sbBuf.append(System.getProperty("line.separator"));
						} else {
							sbBuf.append("]");
						}
					}
					if (j<(tblModel.getColumnCount() - 1)) {
						sbBuf.append(",");
					}
					sbBuf.append(System.getProperty("line.separator"));
					out.write(sbBuf.toString());
				}
				sbBuf.setLength(0);
				sbBuf.append("}");
				sbBuf.append(System.getProperty("line.separator"));
				out.write(sbBuf.toString());

			} else {
				StringBuffer sbBuf = new StringBuffer();
				sbBuf.append("{");
				sbBuf.append(System.getProperty("line.separator"));
				out.write(sbBuf.toString());

				for (int j=0; j<tblModel.getColumnCount(); j++) {

					sbBuf.setLength(0);
					Object objCellVal = tblModel.getValueAt(0,j);

					String strCol = objCellVal.toString();
					if (strCol == null) {
						sbBuf.append("\"\": [");
					} else {
						if ((strCol.indexOf("\"") >= 0) ||
							(strCol.indexOf("\\") >= 0)) {
							// Z̓eɁuhv܂́uv܂܂ꍇ́A
							// ̐擪ɁuvtB
							sbBuf.append("\"");
							for (int m=0; m<strCol.length(); m++) {
								if (strCol.charAt(m) == '"') {
									sbBuf.append("\\\"");
								} else if (strCol.charAt(m) == '\\') {
									sbBuf.append("\\\\");
								} else {
									sbBuf.append(strCol.charAt(m));
								}
							}
							sbBuf.append("\": [");
						} else {
							// JSON`̏ꍇ͐擪Ɩuhvň͂ށB
							sbBuf.append("\"");
							sbBuf.append(strCol);
							sbBuf.append("\": [");
						}
					}
					out.write(sbBuf.toString());
					sbBuf.setLength(0);

					for (int i=1; i<tblModel.getRowCount(); i++) {
						objCellVal = tblModel.getValueAt(i,j);
						if (objCellVal == null) {
							sbBuf.append("\"\"");
						} else {
							if ((objCellVal.toString().indexOf("\"") >= 0) ||
								(objCellVal.toString().indexOf("\\") >= 0)) {
								// Z̓eɁuhv܂́uv܂܂ꍇ́A
								// ̐擪ɁuvtB
								sbBuf.append("\"");
								for (int k=0; k<objCellVal.toString().length(); k++) {
									if (objCellVal.toString().charAt(k) == '"') {
										sbBuf.append("\\\"");
									} else if (objCellVal.toString().charAt(k) == '\\') {
										sbBuf.append("\\\\");
									} else {
										sbBuf.append(objCellVal.toString().charAt(k));
									}
								}
								sbBuf.append("\"");
							} else {
								// JSON`̏ꍇ͐擪Ɩuhvň͂ށB
								sbBuf.append("\"");
								sbBuf.append(objCellVal.toString());
								sbBuf.append("\"");
							}
						}
						if (i<(tblModel.getRowCount() - 1)) {
							sbBuf.append(",");
							sbBuf.append(System.getProperty("line.separator"));
						} else {
							sbBuf.append("]");
						}
					}
					if (j<(tblModel.getColumnCount() - 1)) {
						sbBuf.append(",");
					}
					sbBuf.append(System.getProperty("line.separator"));
					out.write(sbBuf.toString());
				}
				sbBuf.setLength(0);
				sbBuf.append("}");
				sbBuf.append(System.getProperty("line.separator"));
				out.write(sbBuf.toString());

			}

			// Xg[
			out.close();
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}

	////////////////////////////////////////////////////////////////////////////
	//irnm`Ńt@Cóij
	////////////////////////////////////////////////////////////////////////////
	void JsonFileWrite2() {
		try {

			// t@C^Cgɕ\
			setTitle(TITLE + " - " + commonProperties.getFilePath());
			// Xe[^Xo[
			statusBar.setText("m" + commonProperties.getEncode() + "n");

			// t@C
			if (commonProperties.getFilePath() != null) {
				fileChooser.setSelectedFile(new File(commonProperties.getFilePath()));
			}

			// Xg[̍쐬Abv
			OutputStreamWriter out = null;
			if (commonProperties.getEncode().equals("Shift-JIS(Windows{)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "MS932");
			} else if (commonProperties.getEncode().equals("Shift-JIS({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "SJIS");
			} else if (commonProperties.getEncode().equals("EUC({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "EUC_JP");
			} else if (commonProperties.getEncode().equals("JIS({)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "ISO2022JP");
			} else if (commonProperties.getEncode().equals("Unicode(BigEndian)")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "UnicodeBig");
			} else if (commonProperties.getEncode().equals("UTF-8")) {
				out = new OutputStreamWriter(new FileOutputStream(fileChooser.getSelectedFile()), "UTF8");
			}
			// bruwb_L̏ꍇ̓wb_o͂ibrut@CPsڂ̏ój
			if (commonProperties.getCsvHeader().equalsIgnoreCase("1")) {
				StringBuffer sbBuf = new StringBuffer();

				sbBuf.append("[");
				sbBuf.append(System.getProperty("line.separator"));

				sbBuf.append("[");
				sbBuf.append(System.getProperty("line.separator"));

				for (int j=0; j<tblModel.getColumnCount(); j++) {
					String strCol = tblModel.getColumnName(j);
					if (strCol == null) {
						sbBuf.append("\"\"");
					} else {
						if ((strCol.indexOf("\"") >= 0) ||
							(strCol.indexOf("\\") >= 0)) {
							// Z̓eɁuhv܂́uv܂܂ꍇ́A
							// ̐擪ɁuvtB
							sbBuf.append("\"");
							for (int k=0; k<strCol.length(); k++) {
								if (strCol.charAt(k) == '"') {
									sbBuf.append("\\\"");
								} else if (strCol.charAt(k) == '\\') {
									sbBuf.append("\\\\");
								} else {
									sbBuf.append(strCol.charAt(k));
								}
							}
							sbBuf.append("\"");
						} else {
							// JSON`̏ꍇ͐擪Ɩuhvň͂ށB
							sbBuf.append("\"");
							sbBuf.append(strCol);
							sbBuf.append("\"");
						}
					}
					if (j<(tblModel.getColumnCount() - 1)) {
						sbBuf.append(",");
					}
				}
				sbBuf.append(System.getProperty("line.separator"));

				sbBuf.append("],");
				sbBuf.append(System.getProperty("line.separator"));

				out.write(sbBuf.toString());

			} else {
				StringBuffer sbBuf = new StringBuffer();
				sbBuf.append("[");
				sbBuf.append(System.getProperty("line.separator"));
				out.write(sbBuf.toString());
			}
			// Xvbh̊eZ̓et@Cɏo͂ibrut@CQsڈȍ~̏ój
			for (int i=0; i<tblModel.getRowCount(); i++) {
				StringBuffer sbBuf = new StringBuffer();

				sbBuf.append("[");
				sbBuf.append(System.getProperty("line.separator"));

				for (int j=0; j<tblModel.getColumnCount(); j++) {
					Object objCellVal = tblModel.getValueAt(i,j);
					if (objCellVal == null) {
						sbBuf.append("\"\"");
					} else {
						if ((objCellVal.toString().indexOf("\"") >= 0) ||
							(objCellVal.toString().indexOf("\\") >= 0)) {
							// Z̓eɁuhv܂́uv܂܂ꍇ́A
							// ̐擪ɁuvtB
							sbBuf.append("\"");
							for (int k=0; k<objCellVal.toString().length(); k++) {
								if (objCellVal.toString().charAt(k) == '"') {
									sbBuf.append("\\\"");
								} else if (objCellVal.toString().charAt(k) == '\\') {
									sbBuf.append("\\\\");
								} else {
									sbBuf.append(objCellVal.toString().charAt(k));
								}
							}
							sbBuf.append("\"");
						} else {
							// JSON`̏ꍇ͐擪Ɩuhvň͂ށB
							sbBuf.append("\"");
							sbBuf.append(objCellVal.toString());
							sbBuf.append("\"");
						}
					}
					if (j<(tblModel.getColumnCount() - 1)) {
						sbBuf.append(",");
					}
				}
				sbBuf.append(System.getProperty("line.separator"));

				if (i<(tblModel.getRowCount() - 1)) {
					sbBuf.append("],");
					sbBuf.append(System.getProperty("line.separator"));
				} else {
					sbBuf.append("]");
					sbBuf.append(System.getProperty("line.separator"));
				}
				out.write(sbBuf.toString());
			}

			StringBuffer sbBuf = new StringBuffer();
			sbBuf.append("]");
			sbBuf.append(System.getProperty("line.separator"));
			out.write(sbBuf.toString());

			// Xg[
			out.close();
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}

	////////////////////////////////////////////////////////////////////////////
	//e[u̗񕝂̕ύX
	////////////////////////////////////////////////////////////////////////////
	void resizeColumnWidth() {

		int intRow = 0;
		int intCol = 0;
		
		TableColumnModel columnModel = tblArea.getColumnModel();
		for (intCol=0; intCol<tblArea.getColumnCount(); intCol++) {
			int intMaxWidth = 0;
			for (intRow=0; intRow<tblArea.getRowCount(); intRow++) {
				TableCellRenderer rend = tblArea.getCellRenderer(intRow, intCol);
				Object value = tblArea.getValueAt(intRow, intCol);
				Component comp = rend.getTableCellRendererComponent(tblArea,
																	value,
																	false,
																	false,
																	intRow,
																	intCol);
				intMaxWidth = Math.max(comp.getPreferredSize().width, intMaxWidth);
			}
			TableColumn column = columnModel.getColumn(intCol);
			TableCellRenderer headerRend = column.getHeaderRenderer();
			if (headerRend == null) {
				headerRend = tblArea.getTableHeader().getDefaultRenderer();
			}
			Object headerValue = column.getHeaderValue();
			Component headerComp = headerRend.getTableCellRendererComponent(tblArea,
																headerValue,
																false,
																false,
																0,
																intCol);
			intMaxWidth = Math.max(intMaxWidth, headerComp.getPreferredSize().width);
			column.setPreferredWidth(intMaxWidth);
		}
	}

	////////////////////////////////////////////////////////////////////////////
	//C[`
	////////////////////////////////////////////////////////////////////////////
	public static void main(String[] args) {
		new CsvEditor();
	}
}


/******************************************************************************
 *VK쐬_CAO
 ******************************************************************************/
class NewFile extends JDialog implements ActionListener {

	private boolean			bRet = false;
	private JLabel			jlbl1;		// sx
	private JLabel			jlbl2;		// 񐔃x
	private JTextField		jtxtRow;	// seLXg
	private JTextField		jtxtCol;	// 񐔃eLXg

	private JButton			jbtnOK;
	private JButton			jbtnCANCEL;

	private CommonProperties	commonProperties;
	private DefaultTableModel	tblModel;


	////////////////////////////////////////////////////////////////////////////
	//RXgN^
	////////////////////////////////////////////////////////////////////////////
	public NewFile(JFrame parent, CommonProperties commonProp, DefaultTableModel tbl) {

		super(parent, "VK쐬", true);

		bRet = false;

		commonProperties = commonProp;
		tblModel = tbl;

		String strFontName = "lr SVbN";

		getContentPane().setSize(460,160);
		getContentPane().setLayout(null);
		getContentPane().setPreferredSize(new Dimension(460, 160));

		// sx
		jlbl1 = new JLabel("s",JLabel.LEFT);
		jlbl1.setBounds(140,30,40,25);
		jlbl1.setFont(new Font(strFontName,Font.PLAIN | Font.BOLD,12));

		// seLXg
		jtxtRow = new JTextField("10");
		jtxtRow.setBounds(180,30,40,25);
		jtxtRow.setFont(new Font(strFontName,Font.PLAIN,12));
		jtxtRow.setColumns(4);

		// 񐔃x
		jlbl2 = new JLabel("",JLabel.LEFT);
		jlbl2.setBounds(240,30,40,25);
		jlbl2.setFont(new Font(strFontName,Font.PLAIN | Font.BOLD,12));

		// 񐔃eLXg
		jtxtCol = new JTextField("10");
		jtxtCol.setBounds(280,30,40,25);
		jtxtCol.setFont(new Font(strFontName,Font.PLAIN,12));
		jtxtCol.setColumns(4);

		// [VK쐬]{^
		jbtnOK = new JButton("VK쐬");
		jbtnOK.setBounds(145,110,80,25);
		jbtnOK.setFont(new Font(strFontName,Font.PLAIN,12));

		// []{^
		jbtnCANCEL = new JButton("");
		jbtnCANCEL.setBounds(240,110,80,25);
		jbtnCANCEL.setFont(new Font(strFontName,Font.PLAIN,12));

		getContentPane().add(jlbl1);
		getContentPane().add(jlbl2);
		getContentPane().add(jtxtRow);
		getContentPane().add(jtxtCol);
		getContentPane().add(jbtnOK);
		getContentPane().add(jbtnCANCEL);

		jbtnOK.setActionCommand("OkClick");
		jbtnCANCEL.setActionCommand("CancelClick");

		jbtnOK.addActionListener(this);
		jbtnCANCEL.addActionListener(this);

		pack();

		// t[XN[ɕ\
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(((screen.width - getSize().width) / 2),
					((screen.height - getSize().height) / 2));

		setVisible(true);

	}

	////////////////////////////////////////////////////////////////////////////
	//߂l̎擾i[VK쐬]{^[]{^̂ꂪꂽ̔fɎgpj
	////////////////////////////////////////////////////////////////////////////
	public boolean getReturn() {
		return bRet;
	}

	////////////////////////////////////////////////////////////////////////////
	//Cxg
	////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent ae) {
		String cmd = ae.getActionCommand();
		if (cmd.equals("OkClick")) {
			OnOkClick();
		} else if (cmd.equals("CancelClick")) {
			OnCancelClick();
		}
	}

	////////////////////////////////////////////////////////////////////////////
	//[VK쐬]{^NbN
	////////////////////////////////////////////////////////////////////////////
	void OnOkClick() {

		int intRow = 0;
		int intCol = 0;

		// s̃`FbN
		if (jtxtRow.getText().trim().equals("")) {
			JOptionPane.showMessageDialog(this, "s͂Ă܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}
		try {
			intRow = Integer.parseInt(jtxtRow.getText());
			if (intRow < 0) {
				JOptionPane.showMessageDialog(this, "s̒l}CiXlł邽߁AVK쐬̂߂̍sݒł܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
				return;
			}
		} catch(Exception ex) {
			intRow = 0;
			tblModel.setRowCount(0);	// \̑Ss폜
			JOptionPane.showMessageDialog(this, "sɐlȊO͂̕Ă邽߁AVK쐬̂߂̍sݒł܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}

		// 񐔂̃`FbN
		if (jtxtCol.getText().trim().equals("")) {
			JOptionPane.showMessageDialog(this, "񐔂͂Ă܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}
		try {
			intCol = Integer.parseInt(jtxtCol.getText());
			if (intCol < 0) {
				JOptionPane.showMessageDialog(this, "񐔂̒l}CiXlł邽߁AVK쐬̂߂̗񐔂ݒł܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
				return;
			}
		} catch(Exception ex) {
			intCol = 0;
			tblModel.setColumnCount(0);	// \̑S폜
			JOptionPane.showMessageDialog(this, "񐔂ɐlȊO͂̕Ă邽߁AVK쐬̂߂̗񐔂ݒł܂B", "xbZ[W", JOptionPane.WARNING_MESSAGE);
			return;
		}

		tblModel.setRowCount(0);	// \̑Ss폜
		tblModel.setColumnCount(0);	// \̑S폜
		// ̍쐬
		for (int i=0; i<intCol; i++) {
			tblModel.addColumn(Integer.toString(i+1));
		}
		// s̍쐬
		Object[] cell = new Object[intCol];
		for (int i=0; i<intRow; i++) {
			tblModel.addRow(cell);
		}
		
		// 쐬[hi1:VK쐬j
		commonProperties.setCreateMode("1");
		// bruwb_i0:j
		commonProperties.setCsvHeader("0");
		// GR[h
		commonProperties.setEncode("");
		// t@C
		commonProperties.setFilePath(null);
		
		// _CAO\
		setVisible(false);
		
		// ߂l̐ݒ
		bRet = true;
	}

	////////////////////////////////////////////////////////////////////////////
	//[]{^NbN
	////////////////////////////////////////////////////////////////////////////
	void OnCancelClick() {

		// _CAO\
		setVisible(false);

		// ߂l̐ݒ
		bRet = false;
	}
}

/******************************************************************************
 *[wv]-[CsvEditorɂ]
 ******************************************************************************/
class FormAbout extends JDialog implements ActionListener {

	////////////////////////////////////////////////////////////////////////////
	//RXgN^
	////////////////////////////////////////////////////////////////////////////
	public FormAbout(JFrame parent) {

		super(parent, "CsvEditorɂ", true);

		String strFontName = "lr SVbN";

		getContentPane().setSize(550,500);
		getContentPane().setLayout(null);
		getContentPane().setPreferredSize(new Dimension(550, 500));

		// eLXg
		JTextArea jtxtGpl = new JTextArea();
		jtxtGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		JScrollPane scrollpaneGpl = new JScrollPane(jtxtGpl, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollpaneGpl.setBounds(0,0,550,450);
		scrollpaneGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		jtxtGpl.setEditable(false);

		// [nj]{^
		JButton jbtnOk = new JButton("OK");
		jbtnOk.setBounds(245,460,60,30);
		jbtnOk.setFont(new Font(strFontName, Font.PLAIN, 12));
		jbtnOk.setMargin(new Insets(0, 0, 0, 0));
		jbtnOk.setActionCommand("OkClick");
		jbtnOk.addActionListener(this);

		getContentPane().add(scrollpaneGpl);
		getContentPane().add(jbtnOk);

		try {
			InputStream is = getClass().getClassLoader().getResourceAsStream("Resources/Doc/ReadMe.txt");
			//BufferedReader br = new BufferedReader(new InputStreamReader(is,"UTF-8"));
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			jtxtGpl.read(br, null);
			is.close();
			br.close();
		} catch(Exception ex) {
			StringWriter stringWriter = new StringWriter();
			PrintWriter printWriter = new PrintWriter(stringWriter);
			ex.printStackTrace(printWriter);
			ErrorDialog d = new ErrorDialog(new JFrame(), stringWriter.toString());
			//ex.printStackTrace();
		}

		pack();

		// t[XN[ɕ\
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(((screen.width - getSize().width) / 2),
					((screen.height - getSize().height) / 2));

		setVisible(true);
	}

	////////////////////////////////////////////////////////////////////////////
	//Cxg
	////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent ae) {

		String cmd = ae.getActionCommand();

		if (cmd.equals("OkClick")) {
			// _CAO\
			setVisible(false);
		}
	}
}


/******************************************************************************
 *[wv]-[GNU General Public License version 3 (GPLv3)]
 ******************************************************************************/
class FormAboutGpl extends JDialog implements ActionListener {

	////////////////////////////////////////////////////////////////////////////
	//RXgN^
	////////////////////////////////////////////////////////////////////////////
	public FormAboutGpl(JFrame parent) {

		super(parent, "GNU General Public License version 3 (GPLv3)", true);

		String strFontName = "lr SVbN";

		getContentPane().setSize(550,500);
		getContentPane().setLayout(null);
		getContentPane().setPreferredSize(new Dimension(550, 500));

		// eLXg
		JTextArea jtxtGpl = new JTextArea();
		jtxtGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		JScrollPane scrollpaneGpl = new JScrollPane(jtxtGpl, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollpaneGpl.setBounds(0,0,550,450);
		scrollpaneGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		jtxtGpl.setEditable(false);

		// [nj]{^
		JButton jbtnOk = new JButton("OK");
		jbtnOk.setBounds(245,460,60,30);
		jbtnOk.setFont(new Font(strFontName, Font.PLAIN, 12));
		jbtnOk.setMargin(new Insets(0, 0, 0, 0));
		jbtnOk.setActionCommand("OkClick");
		jbtnOk.addActionListener(this);

		getContentPane().add(scrollpaneGpl);
		getContentPane().add(jbtnOk);

		try {
			InputStream is = getClass().getClassLoader().getResourceAsStream("Resources/Doc/gpl-3.0.txt");
			//BufferedReader br = new BufferedReader(new InputStreamReader(is,"UTF-8"));
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			jtxtGpl.read(br, null);
			is.close();
			br.close();
		} catch(Exception ex) {
			StringWriter stringWriter = new StringWriter();
			PrintWriter printWriter = new PrintWriter(stringWriter);
			ex.printStackTrace(printWriter);
			ErrorDialog d = new ErrorDialog(new JFrame(), stringWriter.toString());
			//ex.printStackTrace();
		}

		pack();

		// t[XN[ɕ\
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(((screen.width - getSize().width) / 2),
					((screen.height - getSize().height) / 2));

		setVisible(true);
	}

	////////////////////////////////////////////////////////////////////////////
	//Cxg
	////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent ae) {

		String cmd = ae.getActionCommand();

		if (cmd.equals("OkClick")) {
			// _CAO\
			setVisible(false);
		}
	}
}

/******************************************************************************
 *[wv]-[GNU Lesser General Public License Version 2.1(LGPL2.1)]
 ******************************************************************************/
class FormAboutLgpl extends JDialog implements ActionListener {

	////////////////////////////////////////////////////////////////////////////
	//RXgN^
	////////////////////////////////////////////////////////////////////////////
	public FormAboutLgpl(JFrame parent) {

		super(parent, "GNU Lesser General Public License Version 2.1(LGPL2.1)", true);

		String strFontName = "lr SVbN";

		getContentPane().setSize(550,500);
		getContentPane().setLayout(null);
		getContentPane().setPreferredSize(new Dimension(550, 500));

		// eLXg
		JTextArea jtxtGpl = new JTextArea();
		jtxtGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		JScrollPane scrollpaneGpl = new JScrollPane(jtxtGpl, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollpaneGpl.setBounds(0,0,550,450);
		scrollpaneGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		jtxtGpl.setEditable(false);

		// [nj]{^
		JButton jbtnOk = new JButton("OK");
		jbtnOk.setBounds(245,460,60,30);
		jbtnOk.setFont(new Font(strFontName, Font.PLAIN, 12));
		jbtnOk.setMargin(new Insets(0, 0, 0, 0));
		jbtnOk.setActionCommand("OkClick");
		jbtnOk.addActionListener(this);

		getContentPane().add(scrollpaneGpl);
		getContentPane().add(jbtnOk);

		try {
			InputStream is = getClass().getClassLoader().getResourceAsStream("Resources/Doc/lgpl-2_1.txt");
			//BufferedReader br = new BufferedReader(new InputStreamReader(is,"UTF-8"));
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			jtxtGpl.read(br, null);
			is.close();
			br.close();
		} catch(Exception ex) {
			StringWriter stringWriter = new StringWriter();
			PrintWriter printWriter = new PrintWriter(stringWriter);
			ex.printStackTrace(printWriter);
			ErrorDialog d = new ErrorDialog(new JFrame(), stringWriter.toString());
			//ex.printStackTrace();
		}

		pack();

		// t[XN[ɕ\
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(((screen.width - getSize().width) / 2),
					((screen.height - getSize().height) / 2));

		setVisible(true);
	}

	////////////////////////////////////////////////////////////////////////////
	//Cxg
	////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent ae) {

		String cmd = ae.getActionCommand();

		if (cmd.equals("OkClick")) {
			// _CAO\
			setVisible(false);
		}
	}
}

/******************************************************************************
 *[wv]-[NOTICE]
 ******************************************************************************/
class FormNOTICE extends JDialog implements ActionListener {

	////////////////////////////////////////////////////////////////////////////
	//RXgN^
	////////////////////////////////////////////////////////////////////////////
	public FormNOTICE(JFrame parent) {

		super(parent, "NOTICE", true);

		String strFontName = "lr SVbN";

		getContentPane().setSize(550,500);
		getContentPane().setLayout(null);
		getContentPane().setPreferredSize(new Dimension(550, 500));

		// eLXg
		JTextArea jtxtGpl = new JTextArea();
		jtxtGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		JScrollPane scrollpaneGpl = new JScrollPane(jtxtGpl, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollpaneGpl.setBounds(0,0,550,450);
		scrollpaneGpl.setFont(new Font(strFontName, Font.PLAIN, 12));
		jtxtGpl.setEditable(false);

		// [nj]{^
		JButton jbtnOk = new JButton("OK");
		jbtnOk.setBounds(245,460,60,30);
		jbtnOk.setFont(new Font(strFontName, Font.PLAIN, 12));
		jbtnOk.setMargin(new Insets(0, 0, 0, 0));
		jbtnOk.setActionCommand("OkClick");
		jbtnOk.addActionListener(this);

		getContentPane().add(scrollpaneGpl);
		getContentPane().add(jbtnOk);

		try {
			InputStream is = getClass().getClassLoader().getResourceAsStream("Resources/Doc/NOTICE.txt");
			//BufferedReader br = new BufferedReader(new InputStreamReader(is,"UTF-8"));
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			jtxtGpl.read(br, null);
			is.close();
			br.close();
		} catch(Exception ex) {
			StringWriter stringWriter = new StringWriter();
			PrintWriter printWriter = new PrintWriter(stringWriter);
			ex.printStackTrace(printWriter);
			ErrorDialog d = new ErrorDialog(new JFrame(), stringWriter.toString());
			//ex.printStackTrace();
		}

		pack();

		// t[XN[ɕ\
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(((screen.width - getSize().width) / 2),
					((screen.height - getSize().height) / 2));

		setVisible(true);
	}

	////////////////////////////////////////////////////////////////////////////
	//Cxg
	////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent ae) {

		String cmd = ae.getActionCommand();

		if (cmd.equals("OkClick")) {
			// _CAO\
			setVisible(false);
		}
	}
}

/******************************************************************************
 *G[_CAO\
 ******************************************************************************/
class ErrorDialog extends JDialog implements ActionListener {

	////////////////////////////////////////////////////////////////////////////
	//RXgN^
	////////////////////////////////////////////////////////////////////////////
	public ErrorDialog(JFrame parent, String mes) {

		super(parent, "ERROR!!", true);

		String strFontName = "lr SVbN";

		getContentPane().setSize(550,300);
		getContentPane().setLayout(null);
		getContentPane().setPreferredSize(new Dimension(550, 300));

		// eLXg
		JTextArea jtxtMes = new JTextArea();
		jtxtMes.setFont(new Font(strFontName, Font.PLAIN, 12));
		JScrollPane scrollpaneMes = new JScrollPane(jtxtMes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollpaneMes.setBounds(0,0,550,250);
		scrollpaneMes.setFont(new Font(strFontName, Font.PLAIN, 12));
		jtxtMes.setEditable(false);

		// [nj]{^
		JButton jbtnOk = new JButton("OK");
		jbtnOk.setBounds(245,260,60,30);
		jbtnOk.setFont(new Font(strFontName, Font.PLAIN, 12));
		jbtnOk.setMargin(new Insets(0, 0, 0, 0));
		jbtnOk.setActionCommand("OkClick");
		jbtnOk.addActionListener(this);

		getContentPane().add(scrollpaneMes);
		getContentPane().add(jbtnOk);

		jtxtMes.setText(mes);

		pack();

		// t[XN[ɕ\
		Rectangle screen = getGraphicsConfiguration().getBounds();
		setLocation(((screen.width - getSize().width) / 2),
					((screen.height - getSize().height) / 2));

		setVisible(true);
	}

	////////////////////////////////////////////////////////////////////////////
	//Cxg
	////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent ae) {

		String cmd = ae.getActionCommand();

		if (cmd.equals("OkClick")) {
			// _CAO\
			setVisible(false);
		}
	}
}

/******************************************************************************
 *ʃvpeB
 ******************************************************************************/
class CommonProperties {

	private String csvHeader;		// bruwb_ʁi0:wb_A1:wb_Lj
	private String createMode;		// 쐬[hi0:XVA1:VK쐬j
	private String encode;			// GR[h
	private String filePath;		// t@CitpXj

	// vpeB̑SNA
	public void clear() {
		this.csvHeader = null;
		this.createMode = null;
		this.encode = null;
		this.filePath = null;
		return;
	}
	
	// bruwb_ʂ̎擾
	public String getCsvHeader() {
		return csvHeader;
	}
	// bruwb_ʂ̐ݒ
	public void setCsvHeader(String csvHeader) {
		this.csvHeader = csvHeader;
	}

	// 쐬[h̎擾
	public String getCreateMode() {
		return createMode;
	}
	// 쐬[h̐ݒ
	public void setCreateMode(String createMode) {
		this.createMode = createMode;
	}

	// GR[h̎擾
	public String getEncode() {
		return encode;
	}
	// GR[h̐ݒ
	public void setEncode(String encode) {
		this.encode = encode;
	}

	// t@C̎擾
	public String getFilePath() {
		return filePath;
	}
	// t@C̐ݒ
	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}

}
